@@ -76,7 +76,8 @@ class System(object):
 				"subsystem" : device.subsystem.lower(), 
 				"vendor" : device.vendor.lower(), 
 				"model" : device.model.lower(), 
-				"deviceclass" : device.deviceclass
+				"deviceclass" : device.deviceclass,
+				"driver" : device.driver,
 			}
 
 			# PCI devices provide subsystem information, USB don't.
@@ -114,6 +115,9 @@ class System(object):
 			"public_id" : self.public_id,
 			"private_id" : self.private_id,
 
+			# XXX just for testing:
+			"secret_id" : self._unique_id,
+
 			# Actual profile data
 			"profile" : p,
 		}
@@ -129,8 +133,12 @@ class System(object):
 			This returns a globally (hopefully) ID to identify the host
 			later (by request) in the database.
 		"""
-		return hashlib.sha1(self._unique_id).hexdigest()
-	
+		public_id = self._unique_id
+		if not public_id:
+			return "0" * 40
+
+		return hashlib.sha1(public_id).hexdigest()
+
 	@property
 	def private_id(self):
 		"""
@@ -143,6 +151,9 @@ class System(object):
 		for i in reversed(self._unique_id):
 			private_id += i
 
+		if not private_id:
+			return "0" * 40
+
 		return hashlib.sha1(private_id).hexdigest()
 
 	@property
@@ -153,7 +164,20 @@ class System(object):
 
 			None of the data here is ever sent to the server.
 		"""
-		return "123456789" # XXX just a dummy
+		ids = []
+
+		# Virtual machines (for example) and some boards have a UUID
+		# which is globally unique.
+		for file in ("product_uuid", "product_serial", "chassis_serial"):
+			id = read_from_file(os.path.join(SYS_CLASS_DMI, file)) or ""
+			ids.append(id)
+
+		# As last resort, we use the UUID from pakfire.
+		if not ids:
+			id = read_from_file("/opt/pakfire/db/uuid") or ""
+			ids.append(id)
+
+		return "#".join(ids)
 
 	@property
 	def language(self):
